@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 ###############################################################################
-# $Id: gdal_merge.py 16856 2009-04-26 16:02:36Z hobu $
+# $Id: gdal_merge.py 18953 2010-02-28 12:00:54Z rouault $
 #
 # Project:  InSAR Peppers
 # Purpose:  Module to extract data from many rasters into one output.
@@ -27,12 +27,16 @@
 
 try:
     from osgeo import gdal
+    gdal.TermProgress = gdal.TermProgress_nocb
 except ImportError:
     import gdal
 
-import sys, os, fnmatch
+import sys
+import glob
 
+__version__ = '$id$'[5:-1]
 verbose = 0
+quiet = 0
 
 
 # =============================================================================
@@ -47,9 +51,9 @@ def raster_copy( s_fh, s_xoff, s_yoff, s_xsize, s_ysize, s_band_n,
             nodata )
 
     if verbose != 0:
-        print 'Copy %d,%d,%d,%d to %d,%d,%d,%d.' \
+        print('Copy %d,%d,%d,%d to %d,%d,%d,%d.' \
               % (s_xoff, s_yoff, s_xsize, s_ysize,
-             t_xoff, t_yoff, t_xsize, t_ysize )
+             t_xoff, t_yoff, t_xsize, t_ysize ))
 
     s_band = s_fh.GetRasterBand( s_band_n )
     t_band = t_fh.GetRasterBand( t_band_n )
@@ -72,9 +76,9 @@ def raster_copy_with_nodata( s_fh, s_xoff, s_yoff, s_xsize, s_ysize, s_band_n,
         import Numeric
     
     if verbose != 0:
-        print 'Copy %d,%d,%d,%d to %d,%d,%d,%d.' \
+        print('Copy %d,%d,%d,%d to %d,%d,%d,%d.' \
               % (s_xoff, s_yoff, s_xsize, s_ysize,
-             t_xoff, t_yoff, t_xsize, t_ysize )
+             t_xoff, t_yoff, t_xsize, t_ysize ))
 
     s_band = s_fh.GetRasterBand( s_band_n )
     t_band = t_fh.GetRasterBand( t_band_n )
@@ -102,20 +106,7 @@ def names_to_fileinfos( names ):
     """
     
     file_infos = []
-    i = 0
     for name in names:
-        i += 1
-        # test if the name is a wildcard and not a file
-        if not os.path.isfile(name):
-            if '*' in name:
-                files = os.listdir('.')
-                files.reverse()
-                for file in files:
-                    if fnmatch.fnmatch(file, name):
-                        names.insert(i, file)
-                continue
-            else:
-                raise 'FileIO', ('The file "%s" does not exist' % name)
         fi = file_info()
         if fi.init_from_name( name ) == 1:
             file_infos.append( fi )
@@ -159,13 +150,13 @@ class file_info:
         return 1
 
     def report( self ):
-        print 'Filename: '+ self.filename
-        print 'File Size: %dx%dx%d' \
-              % (self.xsize, self.ysize, self.bands)
-        print 'Pixel Size: %f x %f' \
-              % (self.geotransform[1],self.geotransform[5])
-        print 'UL:(%f,%f)   LR:(%f,%f)' \
-              % (self.ulx,self.uly,self.lrx,self.lry)
+        print('Filename: '+ self.filename)
+        print('File Size: %dx%dx%d' \
+              % (self.xsize, self.ysize, self.bands))
+        print('Pixel Size: %f x %f' \
+              % (self.geotransform[1],self.geotransform[5]))
+        print('UL:(%f,%f)   LR:(%f,%f)' \
+              % (self.ulx,self.uly,self.lrx,self.lry))
 
     def copy_into( self, t_fh, s_band = 1, t_band = 1, nodata_arg=None ):
         """
@@ -242,20 +233,23 @@ class file_info:
 
 # =============================================================================
 def Usage():
-    print 'Usage: gdal_merge.py [-o out_filename] [-of out_format] [-co NAME=VALUE]*'
-    print '                     [-ps pixelsize_x pixelsize_y] [-separate] [-v] [-pct]'
-    print '                     [-ul_lr ulx uly lrx lry] [-n nodata_value] [-init value]'
-    print '                     [-ot datatype] [-createonly] input_files'
-    print '                     [--help-general]'
-    print
+    print('Usage: gdal_merge.py [-o out_filename] [-of out_format] [-co NAME=VALUE]*')
+    print('                     [-ps pixelsize_x pixelsize_y] [-separate] [-q] [-v] [-pct]')
+    print('                     [-ul_lr ulx uly lrx lry] [-n nodata_value] [-init "value [value...]"]')
+    print('                     [-ot datatype] [-createonly] input_files')
+    print('                     [--help-general]')
+    print('')
 
 # =============================================================================
 #
 # Program mainline.
 #
 
-if __name__ == '__main__':
+def main( argv=None ):
 
+    global verbose, quiet
+    verbose = 0
+    quiet = 0
     names = []
     format = 'GTiff'
     out_file = 'out.tif'
@@ -266,12 +260,14 @@ if __name__ == '__main__':
     copy_pct = 0
     nodata = None
     create_options = []
-    pre_init = None
+    pre_init = []
     band_type = None
     createonly = 0
-
+    
     gdal.AllRegister()
-    argv = gdal.GeneralCmdLineProcessor( sys.argv )
+    if argv is None:
+        argv = sys.argv
+    argv = gdal.GeneralCmdLineProcessor( argv )
     if argv is None:
         sys.exit( 0 )
 
@@ -287,6 +283,9 @@ if __name__ == '__main__':
         elif arg == '-v':
             verbose = 1
 
+        elif arg == '-q' or arg == '-quiet':
+            quiet = 1
+
         elif arg == '-createonly':
             createonly = 1
 
@@ -303,12 +302,14 @@ if __name__ == '__main__':
             i = i + 1
             band_type = gdal.GetDataTypeByName( argv[i] )
             if band_type == gdal.GDT_Unknown:
-                print 'Unknown GDAL data type: ', argv[i]
+                print('Unknown GDAL data type: %s' % argv[i])
                 sys.exit( 1 )
 
         elif arg == '-init':
             i = i + 1
-            pre_init = float(argv[i])
+            str_pre_init = argv[i].split()
+            for x in str_pre_init:
+                pre_init.append(float(x))
 
         elif arg == '-n':
             i = i + 1
@@ -340,28 +341,31 @@ if __name__ == '__main__':
             i = i + 4
 
         elif arg[:1] == '-':
-            print 'Unrecognised command option: ', arg
+            print('Unrecognised command option: %s' % arg)
             Usage()
             sys.exit( 1 )
 
         else:
-            names.append( arg )
-            
+            # Expand any possible wildcards from command line arguments
+            f = glob.glob( arg )
+            if len(f) == 0:
+                print('File not found: "%s"' % (str( arg )))
+            names += f # append 1 or more files
         i = i + 1
 
     if len(names) == 0:
-        print 'No input files selected.'
+        print('No input files selected.')
         Usage()
         sys.exit( 1 )
 
     Driver = gdal.GetDriverByName(format)
     if Driver is None:
-        print 'Format driver %s not found, pick a supported driver.' % format
+        print('Format driver %s not found, pick a supported driver.' % format)
         sys.exit( 1 )
 
     DriverMD = Driver.GetMetadata()
-    if not DriverMD.has_key('DCAP_CREATE'):
-        print 'Format driver %s does not support creation and piecewise writing.\nPlease select a format that does, such as GTiff (the default) or HFA (Erdas Imagine).' % format
+    if 'DCAP_CREATE' not in DriverMD:
+        print('Format driver %s does not support creation and piecewise writing.\nPlease select a format that does, such as GTiff (the default) or HFA (Erdas Imagine).' % format)
         sys.exit( 1 )
 
     # Collect information on all the source files.
@@ -406,7 +410,7 @@ if __name__ == '__main__':
         t_fh = Driver.Create( out_file, xsize, ysize, bands,
                               band_type, create_options )
         if t_fh is None:
-            print 'Creation failed, terminating gdal_merge.'
+            print('Creation failed, terminating gdal_merge.')
             sys.exit( 1 )
             
         t_fh.SetGeoTransform( geotransform )
@@ -417,22 +421,37 @@ if __name__ == '__main__':
     else:
         if separate != 0:
             bands = len(file_infos)
+            if t_fh.RasterCount < bands :
+                print('Existing output file has less bands than the number of input files. You should delete it before. Terminating gdal_merge.')
+                sys.exit( 1 )
         else:
             bands = min(file_infos[0].bands,t_fh.RasterCount)
 
     # Do we need to pre-initialize the whole mosaic file to some value?
     if pre_init is not None:
-        for i in range(t_fh.RasterCount):
-            t_fh.GetRasterBand(i+1).Fill( pre_init )
+        if t_fh.RasterCount <= len(pre_init):
+            for i in range(t_fh.RasterCount):
+                t_fh.GetRasterBand(i+1).Fill( pre_init[i] )
+        elif len(pre_init) == 1:
+            for i in range(t_fh.RasterCount):
+                t_fh.GetRasterBand(i+1).Fill( pre_init[0] )
 
     # Copy data from source files into output file.
     t_band = 1
+
+    if quiet == 0 and verbose == 0:
+        gdal.TermProgress( 0.0 )
+    fi_processed = 0
+    
     for fi in file_infos:
         if createonly != 0:
             continue
         
         if verbose != 0:
-            print
+            print("")
+            print("Processing file %5d of %5d, %6.3f%% completed." \
+                  % (fi_processed+1,len(file_infos),
+                     fi_processed * 100.0 / len(file_infos)) )
             fi.report()
 
         if separate == 0 :
@@ -442,5 +461,12 @@ if __name__ == '__main__':
             fi.copy_into( t_fh, 1, t_band, nodata )
             t_band = t_band+1
             
+        fi_processed = fi_processed+1
+        if quiet == 0 and verbose == 0:
+            gdal.TermProgress( fi_processed / float(len(file_infos))  )
+    
     # Force file to be closed.
     t_fh = None
+
+if __name__ == '__main__':
+    sys.exit(main())
